if (typeof Chainable === "undefined") {

    var Chainable = function(customization) {

        // Default options.
        var defaults = {
            defaultaction: 000,
            debug: true,
            prefix: "errorchain.js"
        };
        
        // Keep track of the chain
        var createChain = function() {
            return {
                order:  [], // order of steps
                called: {}  // index steps, for speed
            };
        };
 
        // Log operations by the error chain if in debug mode.
        var log = function() {
            if (options.debug && console && console.log) 
                console.log.apply(console, Array.prototype.slice.call(arguments));
        };

        // Return a default value if obj is undefined.
        var def = function(obj, value) {
            return obj === "undefined" ? value : obj;
        };
        
        // Merge two objects, where obj2 takes precedence.
        var merge = function(obj1, obj2) {
            var obj3 = {};
            for (var attr in obj1) { obj3[attr] = obj1[attr]; }
            for (var attr in obj2) { obj3[attr] = obj2[attr]; }
            return obj3;
        };

        // Merge custom options with defaults.
        var options = merge(defaults, customization);
        
        // No operation, the default action for errors.
        // Reports uncatched errors in debug mode.
        var noop = function(action) { 
            return function(params) {
                var order = params.handler.chain.order;
                log("No action defined for",
                    order[order.length - 1],
                    "Order:", order, params);
            };
        };

        // Default blank parser for data.
        var parse = function(data) {
            return data;
        };
        
        // Check if current chain is circular.
        var isCircularChain = function(action, chain) {
            return action in chain.called; 
        };

        // Parameters passed to each error handling function.
        var params = function(handler, action, data) {
            return {
                action:  action,
                data:    data,
                handler: handler,
                parsed:  handler.parse(data)
            };
        };
    
        // Run the matching error handler, or default error handler.
        // Each handler returns the next step to run, or nothing to stop running.
        var step = function(handler, action, data) {
            return (action in handler) ? 
                handler[action](params(handler, action, data)) : 
                noop(options.defaultaction)(params(handler, action, data));
        };

        // Run the error chain.
        var start = function(action, data) {
            log("Running for action", action, "with data", data);
            
            // Set default values for the action and data.
            action = def(action, options.defaultaction);
            data   = def(data, {});
            
            // Add this call to the chain order
            this.chain.order.push(action);
            
            // Check if this handler has already been called in the chain.
            if(isCircularChain(action, this.chain)) {
                log("Stopping circular chain at action", 
                    action, "-", "Order:", this.chain.order);
                return;
            }
            
            // Store that this handler will be called in this chain.
            this.chain.called[action] = data;
            
            // Run one step in the chain.
            var next = step(this, action, data);

            if(!next) {
                log("Chain completed at action", action);
            }

            // Run the next error handler if needed, or just return true if we're done.
            return next ? start.call(this, next, data) : true;
        };
    
        // Return the created handler with replaceable parts.
        return {
            parse: parse,
            chain: createChain(),
            run: function(action, data) {
                console.log("Starting chain from action", action);
                start.call(this, action, data);
            }
        };

    };

}
