/*
 * ## jquery.console.js
 *
 * Simple JS console for browsers without a built-in solution.
 * 
 * ## Use case:
 * 
 * 1. You need a JS console, but your current browser does not have one, &&
 * 2. You do not have any tools available that can show such a console.
 * 
 * If the window.console object is not set, as in IE7, 
 * this simple jQuery plugin will create a HTML console where messages are displayed.
 *
 * Any script errors will also be displayed, but this will not stop the browser's
 * standard error handler from executing.
 *
 * Do *not* use this plugin in a production setting, as it is pretty ev(i|a)l.
 * This is a development tool. :)
 *
 * ## Usage:
 * 
 * 1. Include jquery, jquery.console.js and jquery.console.css on your site.
 * 2. If the browser in question does not support JSON.stringify, like IE < v8,
 *    you must also include Crockford's JSON parser [1].
 * 3. Initialize the plugin:
 * 
 *     $(function() {
 *         // Create a console inside <body>
 *         $("body").console();
 *         console.log("It works!");
 *     });
 * 
 * There are often better solutions available, 
 * that can give you powerful consoles. 
 * However, in some cases, these tools are not available, 
 * allowed or desirable. This plugin is for these cases.
 * There's a demo in the "test"-folder.
 * 
 * MIT Licensed.
 *
 *     [1]: https://github.com/douglascrockford/JSON-js
 *
 */
(function($) {

    $.fn.extend({

        console: function(customization) {
            
            // Defaults
            var options = $.extend({
                activated: true,             // Should the plugin be activated?
                force: false,                // Always show the console (for debugging of this plugin).
                showOnLoad: false,           // Show the console on page load.
                useNativeErrorHandler: true, // Use the browser's error handler when applicable.
                selector: "consolewrapper",  // Selector used to wrap the console.
                mask: false                  // Do not create the console, but rather a fake console object to prevent errors.
            }, customization);
            
            // Should the plugin be used?
            var active = options.activated && 
                !options.mask && 
                (typeof window.console === "undefined" || options.force);
            
            // Console functions that should be available.
            var consolefunctions = [
                "assert",
                "clear",
                "count",
                "debug",
                "dir",
                "dirxml",
                "error",
                "exception",
                "group",
                "groupCollapsed",
                "groupEnd",
                "info",
                "log",
                "memoryProfile",
                "memoryProfileEnd",
                "profile",
                "profileEnd",
                "table",
                "time",
                "timeEnd",
                "timeStamp",
                "trace",
                "warn"
            ];

            // Handle different types of errors and statements.
            var handler = {
               
                // Catch syntax errors, etc.
                syntax: function() {
                    window.onerror = function(msg, url, line) {
                        console.exception(msg, "|", "URL:", url, "|", "Line:", line);
                        return options.useNativeErrorHandler;
                    };
                },
                
                // Evaluate console input.
                input: function(code) {
                    try {
                        construct.entry("", [eval(code)]);
                    } catch (err) {
                        console.exception(err.name + ":", err.message);
                    }
                },
                
                // Create handlers for normal console statements.
                make: function(level) {
                    return function() {
                        construct.entry(level, 
                            Array.prototype.slice.call(arguments));
                    };
                }

            };
            
            // Attach console to window if need be.
            if(active) {
                $.each(consolefunctions, function(i, fn) {
                    window.console[fn] = handler.make(fn);
                });
            }

            // Mask the console with a fake object to prevent errors.
            if(options.mask) {
                $.each(consolefunctions, function(i, fn) {
                    window.console[fn] = function() {};
                });
            }

            // Find HTML elements created by this plugin.
            var html = {
                container: function()  { return $(options.container); },
                find:      function(s) { return html.container().find(s); },
                wrapper:   function()  { return html.find("#" + options.selector); },
                terminal:  function()  { return html.find(".terminal"); },
                tools:     function()  { return html.find(".tools"); },
                input:     function()  { return html.find("input"); },
                prompt:    function()  { return html.find(".prompt"); }
            };
            
            // Construct needed HTML elements.
            var construct = {
                
                init: function() {
                    construct.wrapper();
                    construct.terminal();
                    construct.tools();
                    construct.filters();
                    construct.prompt();
                    construct.input();
                    construct.exit();
                    construct.bindToggle();
                    html.input().focus();
                },

                bindToggle: function() {
                    $(document).keydown(function(event) {
                        if(event.keyCode == 220 || event.keyCode == 124) {
                            construct.toggle();
                            event.stopPropagation();
                            event.preventDefault();
                        }
                    }); 
                    if(options.showOnLoad) {
                        construct.toggle();
                    }
                },

                toggle: function() {
                    if(html.wrapper().is(":visible")) {
                        html.input().hide();
                        html.prompt().hide();
                        html.wrapper().slideUp("fast");
                        html.input().blur();
                    } else {
                        html.input().fadeIn("slow");
                        html.prompt().fadeIn("slow");
                        html.wrapper().slideDown("fast");
                        html.input().focus();
                    }
                    construct.scroll();
                },

                wrapper: function() {
                    html.container()
                        .append($("<div />")
                        .attr("id", options.selector));
                },

                terminal: function() {
                    html.wrapper()
                        .append($("<div />")
                        .addClass("terminal"));
                },

                tools: function() {
                    html.wrapper()
                        .append($("<div />")
                        .addClass("tools"));
                },

                prompt: function() {
                    html.wrapper()
                        .append($("<div />")
                        .addClass("prompt")
                        .text(">"));
                },

                input: function() {
                    html.wrapper()
                        .append($("<input type='text' />")
                        .keyup(function(event) {
                            if(event.keyCode == 13 && html.input().val() !== "") {
                                var typed = html.input().val();
                                html.input().val("");
                                handler.input(typed); 
                            }
                        }));
                },
                
                exit: function() {
                    html.tools()
                        .append($("<a />")
                        .addClass("exit")
                        .text("exit")
                        .click(function() {
                            construct.toggle();
                        }));
                },

                filters: function() {
                    html.tools()
                        .append($("<a />")
                            .text("clear")    
                            .click(function() { 
                                construct.clear(); }))
                        .append($("<a />")
                            .text("all")
                            .click(function() { 
                                construct.filter("level"); }))
                        .append($("<a />")
                            .text("info")
                            .click(function() { 
                                construct.filter("info");  }))
                        .append($("<a />")
                            .text("debug")
                            .click(function() { 
                                construct.filter("debug");  }))
                        .append($("<a />")
                            .text("warnings")
                            .click(function() { 
                                construct.filter("warn");  }))
                        .append($("<a />")
                            .text("errors")
                            .click(function() { 
                                construct.filter("error"); }))
                        .append($("<a />")
                            .text("exception")
                            .click(function() { 
                                construct.filter("exception");  }));
                },

                entry: function(level, args) {
                    html.terminal()
                        .append($("<div class='level " + level + "' />")
                            .append($("<span>").text(level))
                            .append(construct.stringify(args, " ")));
                    construct.scroll();
                },

                scroll: function() {
                    html.terminal().scrollTop(html.terminal()[0].scrollHeight);
                },

                stringify: function(args) {
                    return $("<div />").addClass("msg")
                        .append($.map(args, function(arg) {
                            return (arg instanceof Object) ? 
                                JSON.stringify(arg) : 
                                construct.parseEmptyString(arg);
                        }).join(" "));
                },

                parseEmptyString: function(str) {
                    return str === "" ? "(an empty string)" : str;
                },

                clear: function() {
                    html.terminal().find(".level").remove();  
                    construct.scroll();
                },

                filter: function(level) {
                    html.terminal().find(".level").hide().filter("." + level).show();
                    construct.scroll();
                }

            };

            return this.each(function() {
                if(!active) return;
                options.container = $(this);
                construct.init();
                handler.syntax(); 
                console.warn("Do not use jquery.console.js in production.");
            });

        }

    });

})(jQuery);
