/*
 * jquery.observe.js
 *
 * Super-simple observation helpers for jQuery.
 *
 * Use case:
 * - You have a literal JS object, and HTML elements which correspond to values in the object.
 * - You'd like updates in the HTML or in the object to be propagated between them.
 * - A full-blown observer library is too much hassle.
 * 
 * Limitations:
 * - Currently only works for input:text, input:checkbox, textarea 
 *   and non-input elements (p, div, etc).
 *   
 * Usage:
 *     
 *     // A simple object with literal fields.
 *     var myobj = { a: 1, b: true, c: "hello" };
 *     
 *     // Changes to these form fields will update the given object and key.
 *     // Input values will be converted to their original type.
 *     $("input:text").observe(myobj, "a");
 *     $("input:checkbox").observe(myobj, "b");
 *
 *     // You can pass a callback function that intercepts values going from HTML to the object.
 *     $("textarea").observe(myobj, "c", function(val) {
 *          return val.toUpperCase();
 *     });
 *     
 *     // If you update the object, you can call its notify function to update the HTML.
 *     // This will overwrite any existing input in the HTML.
 *     myobj.c = "Bye!";
 *     myobj.notify();
 *     
 */
(function($) {

    $.fn.extend({
        
        observe: function(observed, key, callback) {
            
            // Store passed parameters.
            var opts = {
                observed: observed,
                key: key,
                callback: callback
            };

            // Type checking.
            var type = {
                isBoolean:   function(o) { return typeof o === "boolean"; },
                isNumber:    function(o) { return !isNaN(o); },
                isInt:       function(o) { return type.isNumber(o) && o % 1 === 0; },
                isNull:      function(o) { return o === null; },
                isUndefined: function(o) { return typeof o === "undefined"; },
                isBlank:     function(o) { return o === ""; }
            };
            
            // Operations on the value in the object and in the HTML element.
            var value = {
                
                // From HTML to object.
                toObject: function(val) {
                    if(type.isBoolean(opts.original)) return val;

                    if(type.isNumber(opts.original) && type.isBlank(val)) return opts.original;
                    if(type.isInt(opts.original)) return parseInt(val, 10);
                    if(type.isNumber(opts.original)) return parseFloat(val);

                    return val;
                },

                // From object to HTML.
                toHtml: function(val) {
                    if(type.isNull(val) || type.isUndefined(val)) return "";
                    return val;
                },
                
                // Get object value.
                get: function() {
                    return value.toHtml(opts.observed[opts.key]);
                },
                
                // Set object value.
                set: function(val) {
                    opts.observed[opts.key] = type.isUndefined(opts.callback) ?
                        value.toObject(val) : opts.callback(value.toObject(val));
                }

            };
            
            // Handle observation of different HTML element types.
            var handlers = {

                apply: function() {
                    opts.original = value.get();
                    if (opts.container.is("input:text") || opts.container.is("textarea")) {
                        handlers.textfield();
                    } else if (opts.container.is("input:checkbox")) {
                        handlers.checkbox();
                    } else if (opts.container.not("input")) {
                        handlers.element();
                    }
                    opts.container.data("bound", true);
                },
                
                // Add notification method to the observed object,
                // and apply other notifications if they exist.
                chain: function(obj) {
                    if("notify" in opts.observed) {
                        var previous = opts.observed.notify;
                        opts.observed.notify = function() {
                            handlers.apply();
                            previous();
                        };
                    } else {
                        opts.observed.notify = handlers.apply;
                    }
                },

                textfield: function() {
                    opts.container.val(value.get()); 
                    if (opts.container.data("bound")) return;
                    opts.container.bind("keyup change blur click", function() {
                        value.set(opts.container.val());
                    });
                },

                checkbox: function() {
                    if (!type.isBoolean(opts.original)) return;
                    opts.container.prop("checked", value.get());
                    if (opts.container.data("bound")) return;
                    opts.container.bind("keyup change blur click",function() {
                        value.set(opts.container.is(":checked"));
                    });
                },

                element: function() {
                    opts.container.text(value.get());
                }

            };
            
            return this.each(function() {
                opts.container = $(this);
                handlers.chain();
                opts.observed.notify();
            });

        }

    });

})(jQuery);
